package org.gcube.portlets.user.geoportaldataviewer.client;

import java.util.List;

import org.gcube.portlets.user.geoportaldataviewer.shared.GeoNaDataObject;
import org.gcube.portlets.user.geoportaldataviewer.shared.GeoNaDataViewerProfile;
import org.gcube.portlets.user.geoportaldataviewer.shared.gis.BoundsMap;
import org.gcube.portlets.user.geoportaldataviewer.shared.gis.LayerObject;
import org.gcube.portlets.user.geoportaldataviewer.shared.gis.wms.GeoInformationForWMSRequest;
import org.gcube.portlets.user.geoportaldataviewer.shared.products.ConcessioneDV;
import org.gcube.portlets.user.geoportaldataviewer.shared.products.model.UploadedImageDV;

import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

/**
 * The client side stub for the RPC service.
 */
@RemoteServiceRelativePath("geoportaldataviewerservice")
public interface GeoportalDataViewerService extends RemoteService {

	GeoInformationForWMSRequest parseWmsRequest(String wmsRequest, String layerName) throws Exception;

	List<GeoNaDataObject> getDataResult(List<LayerObject> layerObjects, String mapSrsName, BoundsMap mapBBOX,
			int maxWFSFeature, double zoomLevel);

	ConcessioneDV getConcessioneForId(Long id) throws Exception;

	String getMyLogin();

	/**
	 * Gets the layer for type.
	 *
	 * @param layerType the layer type
	 * @return the layer for type
	 * @throws Exception the exception
	 */
	GeoInformationForWMSRequest getLayerForType(String layerType) throws Exception;

	GeoNaDataViewerProfile getGeoNaDataViewProfile() throws Exception;

	/**
	 * Gets the uploaded images for id.
	 *
	 * @param id the id
	 * @return the uploaded images for id
	 * @throws Exception the exception
	 */
	List<UploadedImageDV> getUploadedImagesForId(String itemType, Long id, int maxImages) throws Exception;

}
